from allauth.account import forms as allauth_forms
from allauth.account import views as allauth_views
from django.http import HttpResponse
from django.shortcuts import render
from django.template.loader import render_to_string


def reset_form_errors(form, request, hx_trigger, form_key):
    if request.headers["Hx-Trigger"] == hx_trigger:
        for key in form.errors.keys():
            if key == form_key:
                continue

            if key == "__all__":
                form.errors[key] = form.error_class()
                continue

            if type(form[key].value()) is bool and form[key].value() == False:
                form.errors[key] = form.error_class()
                continue

            if len(form[key].value()) == 0:
                form.errors[key] = form.error_class()

    return form


def htmx_redirect(request, allauth_view):
    response = allauth_view(request)
    response.status_code = 204
    response.headers["HX-Redirect"] = response.url
    return response


def index(request):
    return render(request, "account/index.html")


def login(request):
    if request.method == "POST":
        form = allauth_forms.LoginForm(request.POST, request=request)

        if form.is_valid() and request.headers["Hx-Trigger"] == "submit":
            return htmx_redirect(request, allauth_views.login)

        form = reset_form_errors(form, request, "id_login", "login")

        return HttpResponse(
            render_to_string(
                "account/forms/login_form.html",
                context={"form": form},
                request=request,
            )
        )
    if request.method == "GET":
        return allauth_views.login(request)


def signup(request):
    if request.method == "POST":
        form = allauth_forms.SignupForm(request.POST)

        if form.is_valid() and request.headers["Hx-Trigger"] == "submit":
            return htmx_redirect(request, allauth_views.signup)

        form = reset_form_errors(form, request, "id_email", "email")
        form = reset_form_errors(form, request, "id_password1", "password1")

        return HttpResponse(
            render_to_string(
                "account/forms/signup_form.html",
                context={"form": form},
                request=request,
            )
        )
    if request.method == "GET":
        return allauth_views.signup(request)
